@@ -41,6 +41,11 @@ from nuitka.importing import Importing, Recursion
 from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PostProcessing import executePostProcessing
+from nuitka.Progress import (
+    closeProgressBar,
+    reportProgressBar,
+    setupProgressBar,
+)
 from nuitka.PythonVersions import (
     getPythonABI,
     getSupportedPythonVersions,
@@ -101,7 +106,9 @@ def _createNodeTree(filename):
         removeDirectory(path=standalone_dir, ignore_errors=True)
         makePath(standalone_dir)
 
-    deleteFile(path=OutputDirectories.getResultFullpath(), must_exist=False)
+    deleteFile(
+        path=OutputDirectories.getResultFullpath(onefile=False), must_exist=False
+    )
 
     # Second, do it for the directories given.
     for plugin_filename in Options.getShallFollowExtra():
@@ -294,24 +301,12 @@ def makeSourceDirectory():
         if module.isCompiledPythonModule():
             Finalization.prepareCodeGeneration(module)
 
-    # Pick filenames.
-    source_dir = OutputDirectories.getSourceDirectoryPath()
+    # Do some reporting and determine compiled module to work on
+    compiled_modules = []
 
-    module_filenames = pickSourceFilenames(
-        source_dir=source_dir, modules=ModuleRegistry.getDoneModules()
-    )
-
-    # Generate code for modules.
     for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
-            c_filename = module_filenames[module]
-
-            source_code = CodeGeneration.generateModuleCode(
-                module=module,
-                data_filename=os.path.basename(c_filename + "onst"),  # Really .const
-            )
-
-            writeSourceCode(filename=c_filename, source_code=source_code)
+            compiled_modules.append(module)
 
             if Options.isShowInclusion():
                 inclusion_logger.info(
@@ -332,6 +327,37 @@ def makeSourceDirectory():
         else:
             assert False, module
 
+    # Pick filenames.
+    source_dir = OutputDirectories.getSourceDirectoryPath()
+
+    module_filenames = pickSourceFilenames(
+        source_dir=source_dir, modules=compiled_modules
+    )
+
+    setupProgressBar(
+        stage="C Source Generation",
+        unit="module",
+        total=len(compiled_modules),
+    )
+
+    # Generate code for compiled modules, this can be slow, so do it separately
+    # with a progress bar.
+    for module in compiled_modules:
+        c_filename = module_filenames[module]
+
+        reportProgressBar(
+            item=module.getFullName(),
+        )
+
+        source_code = CodeGeneration.generateModuleCode(
+            module=module,
+            data_filename=os.path.basename(c_filename + "onst"),  # Really .const
+        )
+
+        writeSourceCode(filename=c_filename, source_code=source_code)
+
+    closeProgressBar()
+
     (
         helper_decl_code,
         helper_impl_code,
@@ -371,7 +397,7 @@ def makeSourceDirectory():
 def runSconsBackend(quiet):
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches and statements,
-    # pylint: disable=too-many-branches,too-many-statements
+    # pylint: disable=too-many-branches
 
     asBoolStr = SconsInterface.asBoolStr
 
@@ -398,25 +424,17 @@ def runSconsBackend(quiet):
     }
 
     if not Options.shallMakeModule():
-        options["result_exe"] = OutputDirectories.getResultFullpath()
-
-    # Ask Scons to cache on Windows, except where the directory is thrown
-    # away. On non-Windows you can should use ccache instead.
-    if not Options.isRemoveBuildDir() and Utils.getOS() == "Windows":
-        options["cache_mode"] = "true"
-
-    if Options.isLto():
-        options["lto_mode"] = asBoolStr(True)
+        options["result_exe"] = OutputDirectories.getResultFullpath(onefile=False)
 
     if Options.shallUseStaticLibPython():
         options["static_libpython"] = asBoolStr(True)
 
-    if Options.shallDisableConsoleWindow():
-        options["win_disable_console"] = asBoolStr(True)
-
     if Options.isStandaloneMode():
         options["standalone_mode"] = asBoolStr(True)
 
+    if Options.isOnefileMode():
+        options["onefile_mode"] = asBoolStr(True)
+
     if Options.shallTreatUninstalledPython():
         options["uninstalled_python"] = asBoolStr(True)
 
@@ -425,27 +443,9 @@ def runSconsBackend(quiet):
             len(ModuleRegistry.getUncompiledTechnicalModules())
         )
 
-    if Options.isShowScons():
-        options["show_scons"] = asBoolStr(True)
-
-    if Options.isMingw64():
-        options["mingw_mode"] = asBoolStr(True)
-
-    if Options.getMsvcVersion():
-        msvc_version = Options.getMsvcVersion()
-
-        msvc_version = msvc_version.replace("exp", "Exp")
-        if "." not in msvc_version:
-            msvc_version += ".0"
-
-        options["msvc_version"] = msvc_version
-
     if Utils.getOS() == "Windows":
         options["noelf_mode"] = asBoolStr(True)
 
-    if Options.isClang():
-        options["clang_mode"] = asBoolStr(True)
-
     if Options.isProfile():
         options["profile_mode"] = asBoolStr(True)
 
@@ -485,25 +485,10 @@ def runSconsBackend(quiet):
     if abiflags:
         options["abiflags"] = abiflags
 
-    cpp_defines = Plugins.getPreprocessorSymbols()
-    if cpp_defines:
-        options["cpp_defines"] = ",".join(
-            "%s%s%s" % (key, "=" if value else "", value or "")
-            for key, value in cpp_defines.items()
-        )
-
-    link_libraries = Plugins.getExtraLinkLibraries()
-    if link_libraries:
-        options["link_libraries"] = ",".join(link_libraries)
-
     if Options.shallMakeModule():
         options["module_suffix"] = getSharedLibrarySuffix(preferred=True)
 
-    if Options.shallRunInDebugger():
-        options["full_names"] = asBoolStr(True)
-
-    if Options.assumeYesForDownloads():
-        options["assume_yes_for_downloads"] = asBoolStr(True)
+    SconsInterface.setCommonOptions(options)
 
     return (
         SconsInterface.runScons(
@@ -793,13 +778,18 @@ __name__ = ...
         else:
             general.info("Keeping build directory %r." % source_dir)
 
-        general.info(
-            "Successfully created %r."
-            % OutputDirectories.getResultFullpath(onefile=Options.isOnefileMode())
+        final_filename = OutputDirectories.getResultFullpath(
+            onefile=Options.isOnefileMode()
         )
 
+        Plugins.onFinalResult(final_filename)
+
+        general.info("Successfully created %r." % final_filename)
+
         # Execute the module immediately if option was given.
         if Options.shallExecuteImmediately():
+            general.info("Launching %r." % final_filename)
+
             if Options.shallMakeModule():
                 executeModule(
                     tree=main_module,
@@ -807,8 +797,6 @@ __name__ = ...
                 )
             else:
                 executeMain(
-                    binary_filename=OutputDirectories.getResultFullpath(
-                        onefile=Options.isOnefileMode()
-                    ),
+                    binary_filename=final_filename,
                     clean_path=Options.shallClearPythonPathEnvironment(),
                 )
