@@ -79,6 +79,7 @@ from .ComparisonCodes import (
     generateBuiltinIsinstanceCode,
     generateBuiltinIssubclassCode,
     generateComparisonExpressionCode,
+    generateTypeCheckCode,
 )
 from .ConditionalCodes import (
     generateConditionalAndOrCode,
@@ -544,9 +545,11 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_LOCALS_UPDATED": generateBuiltinLocalsCode,
         "EXPRESSION_BUILTIN_LOCALS_REF": generateBuiltinLocalsRefCode,
         "EXPRESSION_BUILTIN_GLOBALS": generateBuiltinGlobalsCode,
-        "EXPRESSION_BUILTIN_SUPER": generateBuiltinSuperCode,
+        "EXPRESSION_BUILTIN_SUPER0": generateBuiltinSuperCode,
+        "EXPRESSION_BUILTIN_SUPER2": generateBuiltinSuperCode,
         "EXPRESSION_BUILTIN_ISINSTANCE": generateBuiltinIsinstanceCode,
         "EXPRESSION_BUILTIN_ISSUBCLASS": generateBuiltinIssubclassCode,
+        "EXPRESSION_TYPE_CHECK": generateTypeCheckCode,
         "EXPRESSION_BUILTIN_DIR1": generateBuiltinDir1Code,
         "EXPRESSION_BUILTIN_VARS": generateBuiltinVarsCode,
         "EXPRESSION_BUILTIN_HASATTR": generateBuiltinHasattrCode,
@@ -673,9 +676,12 @@ setExpressionDispatchDict(
         "EXPRESSION_OPERATION_INPLACE_BIT_AND": generateOperationBinaryCode,
         "EXPRESSION_OPERATION_INPLACE_BIT_XOR": generateOperationBinaryCode,
         "EXPRESSION_OPERATION_INPLACE_MAT_MULT": generateOperationBinaryCode,
-        "EXPRESSION_OPERATION_UNARY": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_REPR": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_SUB": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_ADD": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_INVERT": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_ABS": generateBuiltinAbsCode,
         "EXPRESSION_OPERATION_NOT": generateOperationNotCode,
-        "EXPRESSION_OPERATION_ABS": generateBuiltinAbsCode,
         "EXPRESSION_OUTLINE_BODY": generateFunctionOutlineCode,
         "EXPRESSION_OUTLINE_FUNCTION": generateFunctionOutlineCode,
         # TODO: Rename to make more clear it is an outline
