@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,12 +29,14 @@ it being used.
 import os
 import shutil
 import sys
+from collections import namedtuple
 
 from nuitka import Options, OutputDirectories
 from nuitka.Errors import NuitkaPluginError
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.SourceCodeReferences import fromFilename
 from nuitka.Tracing import plugins_logger
+from nuitka.utils.Execution import check_output
 from nuitka.utils.FileOperations import putTextFileContents, relpath
 from nuitka.utils.ModuleNames import ModuleName
 
@@ -241,7 +243,7 @@ class NuitkaPluginBase(object):
         Returns:
             dict
         """
-        return self.module_aliases.get(module_name, None)
+        return self.module_aliases.get(module_name)
 
     def onModuleSourceCode(self, module_name, source_code):
         """Inspect or modify source code.
@@ -297,7 +299,7 @@ class NuitkaPluginBase(object):
         return bytecode
 
     @staticmethod
-    def _createTriggerLoadedModule(module, trigger_name, code):
+    def _createTriggerLoadedModule(module, trigger_name, code, flags):
         """Create a "trigger" for a module to be imported.
 
         Notes:
@@ -340,7 +342,8 @@ class NuitkaPluginBase(object):
         if mode == "bytecode":
             trigger_module.setSourceCode(code)
 
-        if Options.is_debug:
+        # In debug mode, put the files in the build folder, so they can be looked up easily.
+        if Options.is_debug and "HIDE_SOURCE" not in flags:
             source_path = os.path.join(
                 OutputDirectories.getSourceDirectoryPath(), module_name + ".py"
             )
@@ -351,7 +354,7 @@ class NuitkaPluginBase(object):
 
     @staticmethod
     def createPreModuleLoadCode(module):
-        """Create code to prepend to a module.
+        """Create code to execute before importing a module.
 
         Notes:
             Called by @onModuleDiscovered.
@@ -359,25 +362,30 @@ class NuitkaPluginBase(object):
         Args:
             module: the module object
         Returns:
+            None (does not apply, default)
             tuple (code, documentary string)
+            tuple (code, documentary string, flags)
         """
         # Virtual method, pylint: disable=unused-argument
-        return None, None
+        return None
 
     @staticmethod
     def createPostModuleLoadCode(module):
-        """Create code to append to a module.
+        """Create code to execute after loading to a module.
 
         Notes:
             Called by @onModuleDiscovered.
 
         Args:
             module: the module object
+
         Returns:
+            None (does not apply, default)
             tuple (code, documentary string)
+            tuple (code, documentary string, flags)
         """
         # Virtual method, pylint: disable=unused-argument
-        return None, None
+        return None
 
     def onModuleDiscovered(self, module):
         """Called with a module to be loaded.
@@ -393,37 +401,60 @@ class NuitkaPluginBase(object):
         Returns:
             None
         """
+
         full_name = module.getFullName()
 
-        pre_code, reason = self.createPreModuleLoadCode(module)
+        preload_desc = self.createPreModuleLoadCode(module)
 
-        if pre_code:
-            # TODO: We could find a way to handle this.
-            if full_name in pre_modules:
-                sys.exit("Error, conflicting plug-ins for %s" % full_name)
+        if preload_desc:
+            if len(preload_desc) == 2:
+                pre_code, reason = preload_desc
+                flags = ()
+            else:
+                pre_code, reason, flags = preload_desc
 
-            self.info("Injecting pre-module load code for module '%s':" % full_name)
-            for line in reason.split("\n"):
-                self.info("    " + line)
+            if pre_code:
+                # Note: We could find a way to handle this if needed.
+                if full_name in pre_modules:
+                    plugins_logger.sysexit(
+                        "Error, conflicting pre module code from plug-ins for %s"
+                        % full_name
+                    )
 
-            pre_modules[full_name] = self._createTriggerLoadedModule(
-                module=module, trigger_name="-preLoad", code=pre_code
-            )
+                self.info("Injecting pre-module load code for module '%s':" % full_name)
+                for line in reason.split("\n"):
+                    self.info("    " + line)
 
-        post_code, reason = self.createPostModuleLoadCode(module)
+                pre_modules[full_name] = self._createTriggerLoadedModule(
+                    module=module, trigger_name="-preLoad", code=pre_code, flags=flags
+                )
 
-        if post_code:
-            # TODO: We could find a way to handle this.
-            if full_name is post_modules:
-                sys.exit("Error, conflicting plug-ins for %s" % full_name)
+        post_desc = self.createPostModuleLoadCode(module)
 
-            self.info("Injecting post-module load code for module '%s':" % full_name)
-            for line in reason.split("\n"):
-                self.info("    " + line)
+        if post_desc:
+            if len(post_desc) == 2:
+                post_code, reason = post_desc
+                flags = ()
+            else:
+                post_code, reason, flags = post_desc
 
-            post_modules[full_name] = self._createTriggerLoadedModule(
-                module=module, trigger_name="-postLoad", code=post_code
-            )
+            if post_code:
+                # Note: We could find a way to handle this if needed.
+                if full_name is post_modules:
+                    plugins_logger.sysexit(
+                        "Error, conflicting post module code from plug-ins for %s"
+                        % full_name
+                    )
+
+                self.info(
+                    "Injecting post-module load code for module '%s':" % full_name
+                )
+                for line in reason.split("\n"):
+                    self.info("    " + line)
+
+                post_modules[full_name] = self._createTriggerLoadedModule(
+                    module=module, trigger_name="-postLoad", code=post_code, flags=flags
+                )
 
     def onModuleEncounter(self, module_filename, module_name, module_kind):
         """Help decide whether to include a module.
@@ -524,9 +555,10 @@ class NuitkaPluginBase(object):
         # TODO: This should no longer be here, as this API is obsolete, pylint: disable=unused-argument
 
         for included_entry_point in self.getExtraDlls(module):
-            # Copy to the dist directory.
-            target_path = (included_entry_point.dest_path,)
-            shutil.copyfile(included_entry_point.source_path, target_path)
+            # Copy to the dist directory, which normally should not be our task, but is for now.
+            shutil.copyfile(
+                included_entry_point.source_path, included_entry_point.dest_path
+            )
 
             yield included_entry_point
 
@@ -568,7 +600,7 @@ class NuitkaPluginBase(object):
         return ()
 
     def onStandaloneDistributionFinished(self, dist_dir):
-        """Called after successfully finishing a standalone compile.
+        """Called after successfully creating a standalone distribution.
 
         Note:
             It is up to the plugin to take subsequent action. Examples are:
@@ -585,6 +617,41 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
 
+    def onOnefileFinished(self, filename):
+        """Called after successfully creating a onefile executable.
+
+        Note:
+            It is up to the plugin to take subsequent action. Examples are:
+            insert additional information (license, copyright, company or
+            application description), create installation material, further
+            folder clean-up, start downstream applications etc.
+
+        Args:
+            filename: the created onefile executable
+
+        Returns:
+            None
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
+    def onFinalResult(self, filename):
+        """Called after successfully finishing a compilation.
+
+        Note:
+            Plugins normally don't need this, and what filename is will be
+            heavily dependent on compilation modes. Actions can be take here,
+            e.g. commercial plugins output generated keys near that executable
+            path.
+        Args:
+            filename: the created binary (module, accelerated exe, dist exe, onefile exe)
+
+        Returns:
+            None
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
     def suppressUnknownImportWarning(self, importing, module_name, source_ref):
         """Suppress import warnings for unknown modules.
 
@@ -674,6 +741,65 @@ class NuitkaPluginBase(object):
                 "Use '--plugin-enable=%s' for: %s" % (self.plugin_name, message)
             )
 
+    _runtime_information_cache = {}
+
+    def queryRuntimeInformationMultiple(self, info_name, setup_codes, values):
+        if info_name in self._runtime_information_cache:
+            return self._runtime_information_cache[info_name]
+
+        keys = []
+        query_codes = []
+
+        for key, value_expression in values:
+            keys.append(key)
+
+            query_codes.append("print(repr(%s))" % value_expression)
+            query_codes.append('print("-" * 27)')
+
+        if type(setup_codes) is str:
+            setup_codes = [setup_codes]
+
+        cmd = r"""\
+from __future__ import print_function
+
+%(setup_codes)s
+%(query_codes)s
+""" % {
+            "setup_codes": "\n".join(setup_codes),
+            "query_codes": "\n".join(query_codes),
+        }
+
+        feedback = check_output([sys.executable, "-c", cmd])
+
+        if str is not bytes:  # We want to work with strings, that's hopefully OK.
+            feedback = feedback.decode("utf8")
+
+        # Ignore Windows newlines difference.
+        feedback = [line.strip() for line in feedback.splitlines()]
+
+        if feedback.count("-" * 27) != len(keys):
+            self.sysexit(
+                "Error, mismatch in output retrieving %r information." % info_name
+            )
+
+        feedback = [line for line in feedback if line != "-" * 27]
+
+        NamedTupleResult = namedtuple(info_name, keys)
+
+        # We are being lazy here, the code is trusted, pylint: disable=eval-used
+        self._runtime_information_cache[info_name] = NamedTupleResult(
+            *(eval(value) for value in feedback)
+        )
+
+        return self._runtime_information_cache[info_name]
+
+    def queryRuntimeInformationSingle(self, setup_codes, value):
+        return self.queryRuntimeInformationMultiple(
+            info_name="temp_info_for_" + self.plugin_name,
+            setup_codes=setup_codes,
+            values=(("key", value),),
+        ).key
+
     @classmethod
     def warning(cls, message):
         plugins_logger.warning(cls.plugin_name + ": " + message)
@@ -682,6 +808,10 @@ class NuitkaPluginBase(object):
     def info(cls, message):
         plugins_logger.info(cls.plugin_name + ": " + message)
 
+    @classmethod
+    def sysexit(cls, message):
+        plugins_logger.sysexit(cls.plugin_name + ": " + message)
+
 
 def isTriggerModule(module):
     return module in pre_modules.values() or module in post_modules.values()
