@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -87,10 +87,10 @@ not exist, a list of all available will be given.""",
             Code to insert and descriptive text (tuple), or (None, None).
         """
         if not isWin32Windows():  # we are only relevant on Windows
-            return None, None
+            return None
 
         if module.getFullName() not in ("PyQt4", "PyQt5"):
-            return None, None  # not for us
+            return None
 
         code = """import os
 path = os.environ.get("PATH", "")
@@ -176,18 +176,19 @@ if os.path.exists(guess_path):
     def considerExtraDlls(self, dist_dir, module):
         # pylint: disable=too-many-branches,too-many-locals,too-many-statements
         full_name = module.getFullName()
-        elements = full_name.split(".")
         plugin_dirs = None
 
-        if elements[0] in ("PyQt4", "PyQt5"):
-            qt_version = int(elements[0][-1])
+        if full_name.getTopLevelPackageName() in ("PyQt4", "PyQt5"):
+            qt_version = int(full_name.getTopLevelPackageName()[-1])
             plugin_dirs = self.getPyQtPluginDirs(qt_version)
 
         if full_name in ("PyQt4", "PyQt5"):
             if not plugin_dirs:
-                sys.exit("Error, failed to detect %s plugin directories." % full_name)
+                self.sysexit(
+                    "Error, failed to detect %s plugin directories." % full_name
+                )
 
-            target_plugin_dir = os.path.join(dist_dir, full_name, "qt-plugins")
+            target_plugin_dir = os.path.join(dist_dir, full_name.asPath(), "qt-plugins")
 
             plugin_options = set(self.qt_plugins.split(","))
 
@@ -236,7 +237,7 @@ if os.path.exists(guess_path):
                     if not os.path.isdir(
                         os.path.join(target_plugin_dir, plugin_candidate)
                     ):
-                        sys.exit(
+                        self.sysexit(
                             "Error, no such Qt plugin family: %s" % plugin_candidate
                         )
 
@@ -285,7 +286,7 @@ if os.path.exists(guess_path):
                     if os.path.exists(qml_plugin_dir):
                         break
                 else:
-                    sys.exit("Error, no such Qt plugin family: qml")
+                    self.sysexit("Error, no such Qt plugin family: qml")
 
                 qml_target_dir = os.path.normpath(
                     os.path.join(target_plugin_dir, "..", "Qt", "qml")
@@ -352,7 +353,15 @@ if os.path.exists(guess_path):
                     shutil.copy(dll_path, dist_dll_path)
 
         elif (
-            full_name.startswith(("PyQt4.QtWebEngine", "PyQt5.QtWebEngine"))
+            full_name
+            in (
+                "PyQt4.QtWebEngine",
+                "PyQt5.QtWebEngine",
+                "PyQt4.QtWebEngineCore",
+                "PyQt5.QtWebEngineCore",
+                "PyQt4.QtWebEngineWidgets",
+                "PyQt5.QtWebEngineWidgets",
+            )
             and not self.webengine_done
         ):
             self.webengine_done = True  # prevent multiple copies
@@ -377,7 +386,7 @@ if os.path.exists(guess_path):
             translations_dir = os.path.join(plugin_parent, "translations")
             pos = len(translations_dir) + 1
             target_translations_dir = os.path.join(
-                dist_dir, elements[0], "Qt", "translations"
+                dist_dir, full_name.getTopLevelPackageName(), "Qt", "translations"
             )
             for f in getFileList(translations_dir):
                 tar_f = os.path.join(target_translations_dir, f[pos:])
@@ -426,12 +435,10 @@ if os.path.exists(guess_path):
         full_name = module.getFullName()
 
         if full_name in ("PyQt4.QtCore", "PyQt5.QtCore"):
-            qt_version = int(full_name.split(".")[0][-1])
-
             code = """\
 from __future__ import absolute_import
 
-from PyQt%(qt_version)d.QtCore import QCoreApplication
+from %(package_name)s import QCoreApplication
 import os
 
 QCoreApplication.setLibraryPaths(
@@ -443,7 +450,7 @@ QCoreApplication.setLibraryPaths(
     ]
 )
 """ % {
-                "qt_version": qt_version
+                "package_name": full_name
             }
 
             return (
@@ -453,8 +460,6 @@ Setting Qt library path to distribution folder. Need to avoid loading target
 system Qt plug-ins, which may be from another Qt version.""",
             )
 
-        return None, None
-
 
 class NuitkaPluginDetectorPyQtPySidePlugins(NuitkaPluginBase):
     detector_for = NuitkaPluginPyQtPySidePlugins
