@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,20 +22,15 @@
 import __future__
 
 import ast
-from logging import warning
 
-from nuitka import Constants, Options, Tracing
+from nuitka import Constants, Options
 from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
-from nuitka.nodes.ContainerMakingNodes import (
-    ExpressionMakeList,
-    ExpressionMakeSetLiteral,
-    ExpressionMakeTuple,
-)
+from nuitka.nodes.ContainerMakingNodes import makeExpressionMakeTupleOrConstant
 from nuitka.nodes.DictionaryNodes import (
     ExpressionKeyValuePair,
-    ExpressionMakeDict,
+    makeExpressionMakeDict,
 )
 from nuitka.nodes.ExceptionNodes import StatementReraiseException
 from nuitka.nodes.FrameNodes import (
@@ -45,18 +40,15 @@ from nuitka.nodes.FrameNodes import (
     StatementsFrameGenerator,
     StatementsFrameModule,
 )
-from nuitka.nodes.ImportNodes import ExpressionBuiltinImport
 from nuitka.nodes.NodeBases import NodeBase
 from nuitka.nodes.NodeMakingHelpers import mergeStatements
 from nuitka.nodes.StatementNodes import StatementsSequence
-from nuitka.PythonVersions import (
-    needsSetLiteralReverseInsertion,
-    python_version,
-)
+from nuitka.PythonVersions import python_version
+from nuitka.Tracing import optimization_logger, printLine
 
 
 def dump(node):
-    Tracing.printLine(ast.dump(node))
+    printLine(ast.dump(node))
 
 
 def getKind(node):
@@ -68,7 +60,7 @@ def extractDocFromBody(node):
     doc = None
 
     # Work around ast.get_docstring breakage.
-    if node.body and getKind(body[0]) == "Expr":
+    if body and getKind(body[0]) == "Expr":
         if getKind(body[0].value) == "Str":  # python3.7 or earlier
             doc = body[0].value.s
             body = body[1:]
@@ -131,9 +123,9 @@ def detectFunctionBodyKind(nodes, start_value=None):
 
         if node_class is ast.Yield:
             indications.add("Generator")
-        elif python_version >= 300 and node_class is ast.YieldFrom:
+        elif python_version >= 0x300 and node_class is ast.YieldFrom:
             indications.add("Generator")
-        elif python_version >= 350 and node_class in (ast.Await, ast.AsyncWith):
+        elif python_version >= 0x350 and node_class in (ast.Await, ast.AsyncWith):
             indications.add("Coroutine")
 
         # Recurse to children, but do not cross scope boundary doing so.
@@ -153,7 +145,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                 else:
                     assert False, (name, field, ast.dump(node))
         elif node_class in (ast.FunctionDef, ast.Lambda) or (
-            python_version >= 350 and node_class is ast.AsyncFunctionDef
+            python_version >= 0x350 and node_class is ast.AsyncFunctionDef
         ):
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body"):
@@ -165,7 +157,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     for child in field.defaults:
                         _check(child)
 
-                    if python_version >= 300:
+                    if python_version >= 0x300:
                         for child in node.args.kw_defaults:
                             if child is not None:
                                 _check(child)
@@ -187,7 +179,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                 if name == "name":
                     pass
                 elif name in ("body", "comparators", "elt"):
-                    if python_version >= 370:
+                    if python_version >= 0x370:
                         _checkCoroutine(field)
                 elif name == "generators":
                     _check(field[0].iter)
@@ -195,7 +187,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     # New syntax in 3.7 allows these to be present in functions not
                     # declared with "async def", so we need to check them, but
                     # only if top level.
-                    if python_version >= 370 and node in nodes:
+                    if python_version >= 0x370 and node in nodes:
                         for gen in field:
                             if gen.is_async:
                                 indications.add("Coroutine")
@@ -205,7 +197,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                                 break
                 else:
                     assert False, (name, field, ast.dump(node))
-        elif node_class is ast.ListComp and python_version >= 300:
+        elif node_class is ast.ListComp and python_version >= 0x300:
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body", "comparators"):
                     pass
@@ -215,7 +207,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     _check(field)
                 else:
                     assert False, (name, field, ast.dump(node))
-        elif python_version >= 270 and node_class is ast.SetComp:
+        elif python_version >= 0x270 and node_class is ast.SetComp:
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body", "comparators", "elt"):
                     pass
@@ -223,7 +215,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     _check(field[0].iter)
                 else:
                     assert False, (name, field, ast.dump(node))
-        elif python_version >= 270 and node_class is ast.DictComp:
+        elif python_version >= 0x270 and node_class is ast.DictComp:
             for name, field in ast.iter_fields(node):
                 if name in ("name", "body", "comparators", "key", "value"):
                     pass
@@ -231,7 +223,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
                     _check(field[0].iter)
                 else:
                     assert False, (name, field, ast.dump(node))
-        elif python_version >= 370 and node_class is ast.comprehension:
+        elif python_version >= 0x370 and node_class is ast.comprehension:
             for name, field in ast.iter_fields(node):
                 if name in ("name", "target"):
                     pass
@@ -248,9 +240,9 @@ def detectFunctionBodyKind(nodes, start_value=None):
                 else:
                     assert False, (name, field, ast.dump(node))
         elif node_class is ast.Name:
-            if python_version >= 300 and node.id == "super":
+            if python_version >= 0x300 and node.id == "super":
                 flags.add("has_super")
-        elif python_version < 300 and node_class is ast.Exec:
+        elif python_version < 0x300 and node_class is ast.Exec:
             flags.add("has_exec")
 
             if node.globals is None:
@@ -258,7 +250,7 @@ def detectFunctionBodyKind(nodes, start_value=None):
 
             for child in ast.iter_child_nodes(node):
                 _check(child)
-        elif python_version < 300 and node_class is ast.ImportFrom:
+        elif python_version < 0x300 and node_class is ast.ImportFrom:
             for import_desc in node.names:
                 if import_desc.name[0] == "*":
                     flags.add("has_exec")
@@ -336,7 +328,9 @@ def buildNode(provider, node, source_ref, allow_none=False):
         # code exception, don't warn about it with a code dump then.
         raise
     except:
-        warning("Problem at '%s' with %s." % (source_ref, ast.dump(node)))
+        optimization_logger.warning(
+            "Problem at '%s' with %s." % (source_ref, ast.dump(node))
+        )
         raise
 
 
@@ -365,7 +359,7 @@ _host_node = None
 
 def buildAnnotationNode(provider, node, source_ref):
     if (
-        python_version >= 370
+        python_version >= 0x370
         and provider.getParentModule().getFutureSpec().isFutureAnnotations()
     ):
 
@@ -407,7 +401,7 @@ def buildAnnotationNode(provider, node, source_ref):
 def makeModuleFrame(module, statements, source_ref):
     assert module.isCompiledPythonModule()
 
-    if Options.isFullCompat():
+    if Options.is_fullcompat:
         code_name = "<module>"
     else:
         if module.isMainModule():
@@ -421,6 +415,7 @@ def makeModuleFrame(module, statements, source_ref):
             co_name=code_name,
             co_kind="Module",
             co_varnames=(),
+            co_freevars=(),
             co_argcount=0,
             co_posonlyargcount=0,
             co_kwonlyargcount=0,
@@ -540,110 +535,6 @@ def makeStatementsSequenceFromStatements(*statements):
     )
 
 
-def makeSequenceCreationOrConstant(sequence_kind, elements, source_ref):
-    # Sequence creation. Tries to avoid creations with only constant
-    # elements. Would be caught by optimization, but would be useless churn. For
-    # mutable constants we cannot do it though.
-
-    # Due to the many sequence types, there is a lot of cases here
-    # pylint: disable=too-many-branches
-
-    for element in elements:
-        if not element.isExpressionConstantRef():
-            constant = False
-            break
-    else:
-        constant = True
-
-    sequence_kind = sequence_kind.lower()
-
-    # Note: This would happen in optimization instead, but lets just do it
-    # immediately to save some time.
-    if constant:
-        if sequence_kind == "tuple":
-            const_type = tuple
-        elif sequence_kind == "list":
-            const_type = list
-        elif sequence_kind == "set":
-            const_type = set
-
-            if needsSetLiteralReverseInsertion():
-                elements = tuple(reversed(elements))
-        else:
-            assert False, sequence_kind
-
-        result = makeConstantRefNode(
-            constant=const_type(element.getConstant() for element in elements),
-            source_ref=source_ref,
-            user_provided=True,
-        )
-    else:
-        if sequence_kind == "tuple":
-            result = ExpressionMakeTuple(elements=elements, source_ref=source_ref)
-        elif sequence_kind == "list":
-            result = ExpressionMakeList(elements=elements, source_ref=source_ref)
-        elif sequence_kind == "set":
-            result = ExpressionMakeSetLiteral(elements=elements, source_ref=source_ref)
-        else:
-            assert False, sequence_kind
-
-    if elements:
-        result.setCompatibleSourceReference(
-            source_ref=elements[-1].getCompatibleSourceReference()
-        )
-
-    return result
-
-
-def makeDictCreationOrConstant(keys, values, source_ref):
-    # Create dictionary node. Tries to avoid it for constant values that are not
-    # mutable.
-
-    assert len(keys) == len(values)
-    for key, value in zip(keys, values):
-        if not key.isExpressionConstantRef() or not key.isKnownToBeHashable():
-            constant = False
-            break
-
-        if not value.isExpressionConstantRef():
-            constant = False
-            break
-    else:
-        constant = True
-
-    # Note: This would happen in optimization instead, but lets just do it
-    # immediately to save some time.
-    if constant:
-        # Unless told otherwise, create the dictionary in its full size, so
-        # that no growing occurs and the constant becomes as similar as possible
-        # before being marshaled.
-        result = makeConstantRefNode(
-            constant=Constants.createConstantDict(
-                keys=[key.getConstant() for key in keys],
-                values=[value.getConstant() for value in values],
-            ),
-            user_provided=True,
-            source_ref=source_ref,
-        )
-    else:
-        result = ExpressionMakeDict(
-            pairs=[
-                ExpressionKeyValuePair(
-                    key=key, value=value, source_ref=key.getSourceReference()
-                )
-                for key, value in zip(keys, values)
-            ],
-            source_ref=source_ref,
-        )
-
-    if values:
-        result.setCompatibleSourceReference(
-            source_ref=values[-1].getCompatibleSourceReference()
-        )
-
-    return result
-
-
 def makeDictCreationOrConstant2(keys, values, source_ref):
     # Create dictionary node. Tries to avoid it for constant values that are not
     # mutable. Keys are Python strings here.
@@ -664,13 +555,13 @@ def makeDictCreationOrConstant2(keys, values, source_ref):
         # before being marshaled.
         result = makeConstantRefNode(
             constant=Constants.createConstantDict(
-                keys=keys, values=[value.getConstant() for value in values]
+                keys=keys, values=[value.getCompileTimeConstant() for value in values]
             ),
             user_provided=True,
             source_ref=source_ref,
         )
     else:
-        result = ExpressionMakeDict(
+        result = makeExpressionMakeDict(
             pairs=[
                 ExpressionKeyValuePair(
                     key=makeConstantRefNode(
@@ -711,46 +602,34 @@ def getStatementsPrepended(statement_sequence, statements):
 
 
 def makeReraiseExceptionStatement(source_ref):
-    # TODO: Remove the statement sequence packaging and have users do it themselves
-    # in factory functions instead.
-
-    return StatementsSequence(
-        statements=(StatementReraiseException(source_ref=source_ref),),
-        source_ref=source_ref,
-    )
+    return StatementReraiseException(source_ref=source_ref)
 
 
-def makeAbsoluteImportNode(module_name, source_ref):
-    return ExpressionBuiltinImport(
-        name=makeConstantRefNode(module_name, source_ref, True),
-        globals_arg=None,
-        locals_arg=None,
-        fromlist=None,
-        level=makeConstantRefNode(0, source_ref, True),
-        source_ref=source_ref,
-    )
+def mangleName(name, owner):
+    """Mangle names with leading "__" for usage in a class owner.
 
+    Notes: The is the private name handling for Python classes.
+    """
 
-def mangleName(variable_name, owner):
-    if not variable_name.startswith("__") or variable_name.endswith("__"):
-        return variable_name
+    if not name.startswith("__") or name.endswith("__"):
+        return name
     else:
         # The mangling of function variable names depends on being inside a
         # class.
         class_container = owner.getContainingClassDictCreation()
 
         if class_container is None:
-            return variable_name
+            return name
         else:
-            return "_%s%s" % (class_container.getName().lstrip("_"), variable_name)
+            return "_%s%s" % (class_container.getName().lstrip("_"), name)
 
 
 def makeCallNode(called, *args, **kwargs):
     source_ref = args[-1]
 
     if len(args) > 1:
-        args = makeSequenceCreationOrConstant(
-            sequence_kind="tuple", elements=args[:-1], source_ref=source_ref
+        args = makeExpressionMakeTupleOrConstant(
+            elements=args[:-1], user_provided=True, source_ref=source_ref
         )
     else:
         args = None
